"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const color_1 = require("../color");
const log_1 = require("../log");
const utils_1 = require("../utils");
class Container {
    constructor(container) {
        this.container = container;
    }
    stat() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.container) {
                throw new Error("container is null.");
            }
            const stream = yield this.container.stats();
            return new StatsStream(stream);
        });
    }
    inspect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.container) {
                throw new Error("container is null.");
            }
            try {
                const data = yield this.container.inspect();
                return JSON.stringify(data, null, 2);
            }
            catch (error) {
                log_1.Log.error(error);
            }
            return "";
        });
    }
}
exports.default = Container;
class StatsStream {
    constructor(stream) {
        this.stream = stream;
    }
    ondata(callback) {
        if (this.stream) {
            const cpuUsage = new CPUUsage();
            const memUsage = new MemoryUsage();
            const txData = new TXData();
            const rxData = new RXData();
            this.stream.on('data', (chunk) => {
                try {
                    const stat = JSON.parse(chunk.toString());
                    cpuUsage.push(stat);
                    memUsage.push(stat);
                    txData.push(stat);
                    rxData.push(stat);
                    callback(cpuUsage, memUsage, txData, rxData);
                }
                catch (error) {
                    log_1.Log.error(error);
                    this.end();
                }
            });
        }
    }
    end() {
        if (this.stream) {
            this.stream.emit('end');
        }
    }
}
exports.StatsStream = StatsStream;
class Usage {
    constructor() {
        this.MAX_DATA_LENGTH = 20;
        this.X = new Array();
        this.Y = new Array();
    }
    get x() {
        return this.X;
    }
    get y() {
        return this.Y;
    }
    shift() {
        if (this.y.length >= this.MAX_DATA_LENGTH) {
            this.y.shift();
        }
        if (this.x.length >= this.MAX_DATA_LENGTH) {
            this.x.shift();
        }
    }
}
exports.Usage = Usage;
Usage.EMPTY = { x: [], y: [] };
class CPUUsage extends Usage {
    push(stat) {
        const time = utils_1.toLocalTime(stat.read);
        let total = stat.cpu_stats.cpu_usage.total_usage - stat.precpu_stats.cpu_usage.total_usage;
        let system = stat.cpu_stats.system_cpu_usage - stat.precpu_stats.system_cpu_usage;
        let num = 1;
        if (stat.cpu_stats.cpu_usage.percpu_usage) {
            num = stat.cpu_stats.cpu_usage.percpu_usage.length;
        }
        if (system > 0.0) {
            let percent = (total / system) * num * 100;
            this.shift();
            this.y.push(percent);
            log_1.Log.info(time);
            this.x.push(time);
        }
    }
}
exports.CPUUsage = CPUUsage;
class RXData extends Usage {
    get title() {
        return "RX";
    }
    get style() {
        return {
            line: color_1.Color.magenta
        };
    }
    push(stat) {
        const time = utils_1.toLocalTime(stat.read);
        let preValue = 0;
        if (this.y.length > 0) {
            preValue = this.y[this.y.length - 1];
        }
        var rxData = (stat.networks.eth0.rx_bytes - preValue) / 1024;
        this.shift();
        this.y.push(Number(rxData.toFixed(0)));
        this.x.push(time);
    }
}
exports.RXData = RXData;
class TXData extends Usage {
    get title() {
        return "TX";
    }
    get style() {
        return {
            line: color_1.Color.yellow
        };
    }
    push(stat) {
        const time = utils_1.toLocalTime(stat.read);
        let preValue = 0;
        if (this.y.length > 0) {
            preValue = this.y[this.y.length - 1];
        }
        var txData = (stat.networks.eth0.tx_bytes - preValue) / 1024;
        this.shift();
        this.y.push(Number(txData.toFixed(0)));
        this.x.push(time);
    }
}
exports.TXData = TXData;
exports.EMPTY_NET_DATA = [new RXData(), new TXData()];
class MemoryUsage extends Usage {
    push(stat) {
        this.shift();
        const time = utils_1.toLocalTime(stat.read);
        this.y.push(stat.memory_stats.usage / 1024 / 1024);
        this.x.push(time);
    }
}
exports.MemoryUsage = MemoryUsage;
