"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const color_1 = require("../color");
const log_1 = require("../log");
const utils_1 = require("../utils");
const container_1 = __importDefault(require("./container"));
const image_1 = __importDefault(require("./image"));
const Docker = require('dockerode');
class Dockerode {
    constructor() {
        this.docker = new Docker({
            socketPath: '/var/run/docker.sock'
        });
    }
    static get singleton() {
        if (!this.instance) {
            this.instance = new Dockerode();
        }
        return this.instance;
    }
    getContainer(selectId) {
        return new container_1.default(this.docker.getContainer(selectId));
    }
    getImage(selectId) {
        return new image_1.default(this.docker.getImage(selectId));
    }
    getDockerVersion() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = [];
            try {
                const version = yield this.docker.version();
                if (!version) {
                    return data;
                }
                data.push(['Docker version', version.Version]);
                data.push(['Docker api version', version.ApiVersion]);
                data.push(['Go version', version.GoVersion]);
                data.push(['Build', version.GitCommit]);
                data.push(['Build time', utils_1.toLocalDateTime(version.BuildTime)]);
            }
            catch (error) {
                log_1.Log.error(error);
                data.push(['Error', error.errno]);
            }
            return data;
        });
    }
    getStatistics() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = [['Containers', 'Images', 'Networks', 'Volumes']];
            try {
                const info = yield this.docker.info();
                if (!info) {
                    return data;
                }
                const running = 'Running : ' + color_1.ColorText.cyan(info.ContainersRunning.toString());
                const stopped = 'Stopped : ' + color_1.ColorText.red(info.ContainersStopped.toString());
                const containers = info.Containers.toString() + ' ( ' + running + ' , ' + stopped + ' )';
                const images = yield this.totalImages();
                const nets = yield this.statistic(() => this.docker.listNetworks());
                const volumes = yield this.statistic(() => __awaiter(this, void 0, void 0, function* () {
                    const result = yield this.docker.listVolumes();
                    if (result) {
                        return result.Volumes;
                    }
                }));
                data.push([containers, images, nets, volumes]);
            }
            catch (error) {
                log_1.Log.error(error);
                data.push([error.errno]);
            }
            return data;
        });
    }
    statistic(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield callback();
                if (!result) {
                    return '-';
                }
                return result.length ? result.length.toString() : '0';
            }
            catch (error) {
                log_1.Log.error(error);
                return error.errno;
            }
        });
    }
    totalImages() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const images = yield this.docker.listImages();
                if (!images) {
                    return '-';
                }
                const length = images.length ? images.length.toString() : '0';
                return length + ' ( Size : ' + this.sizeOf(images) + ' GB )';
            }
            catch (error) {
                log_1.Log.error(error);
                return error.errno;
            }
        });
    }
    listImages() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = [['Id', 'Repository', 'Tag', 'Size', 'Created']];
            try {
                const images = yield this.docker.listImages();
                if (!images) {
                    return data;
                }
                images.forEach((image) => {
                    const repoTag = this.getRepoTag(image.RepoTags);
                    const size = (image.Size / 1000 / 1000).toFixed(2) + ' MB';
                    const created = utils_1.fromNow(image.Created);
                    data.push([this.getId(image.Id), repoTag.repo, repoTag.tag, size, created]);
                });
            }
            catch (error) {
                log_1.Log.error(error);
            }
            return data;
        });
    }
    getRepoTag(repoTags) {
        const repo = {
            repo: '',
            tag: ''
        };
        if (repoTags && repoTags.length && repoTags.length > 0) {
            let tmp = repoTags[0].toString();
            let index = tmp.lastIndexOf(':');
            if (index >= 0) {
                repo.repo = tmp.substring(0, index);
                repo.tag = tmp.substring(index + 1, tmp.length);
            }
            else {
                repo.repo = tmp;
            }
        }
        return repo;
    }
    getId(id) {
        if (!id)
            return '-';
        let index = id.indexOf(':');
        if (index >= 0) {
            return id.substring(index + 1, index + 13);
        }
        return id.substring(0, 12);
    }
    sizeOf(images) {
        const total = images.reduce((total, image) => total + image.Size / 1000 / 1000, 0);
        return (total / 1000).toFixed(2);
    }
    listContainers() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = [['Id', 'Name', 'Image', 'IP', 'Ports', 'State']];
            try {
                const containers = yield this.docker.listContainers({ all: true });
                if (!containers) {
                    return data;
                }
                containers.forEach((container) => {
                    data.push(this.toRow(container));
                });
            }
            catch (error) {
                log_1.Log.error(error);
            }
            return data;
        });
    }
    toRow(container) {
        const row = [];
        row.push(container.Id.substring(0, 8));
        if (container.Names.length > 0) {
            var name = container.Names[0];
            row.push(name.substring(1, name.length));
        }
        else {
            row.push('-');
        }
        row.push(container.Image);
        if (container.State == 'running') {
            let mode = container.HostConfig.NetworkMode;
            row.push(container.NetworkSettings.Networks[mode == 'default' ? 'bridge' : mode].IPAddress);
            if (container.Ports.length > 0) {
                const ports = container.Ports.map(port => port.PrivatePort + ':' + port.PublicPort).join(' ');
                row.push(ports.toString());
            }
            else {
                row.push('-');
            }
            row.push(color_1.ColorText.RUNNING);
        }
        else {
            row.push('-');
            row.push('-');
            row.push(color_1.ColorText.state(container.State));
        }
        return row;
    }
    ;
    listNetworks() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = [['Name', 'Scope', 'Driver', 'Internal', 'IPAM Driver', 'IPAM Subnet', 'IPAM Gateway']];
            try {
                const nets = yield this.docker.listNetworks();
                nets.forEach((net) => {
                    const row = [net.Name, net.Scope, net.Driver, net.Internal];
                    if (net.IPAM) {
                        row.push(net.IPAM.Driver);
                        if (net.IPAM.Config.length > 0) {
                            var c = net.IPAM.Config[0];
                            row.push(!c.Subnet ? '-' : c.Subnet.trim());
                            row.push(!c.Gateway ? '-' : c.Gateway.trim());
                        }
                    }
                    data.push(row);
                });
            }
            catch (error) {
                log_1.Log.error(error);
            }
            return data;
        });
    }
    listVolumes() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = [['Name', 'Driver', 'Mountpoint']];
            try {
                const result = yield this.docker.listVolumes();
                if (!result || !result.Volumes) {
                    return data;
                }
                result.Volumes.forEach((volume) => data.push([volume.Name, volume.Driver, volume.Mountpoint]));
            }
            catch (error) {
                log_1.Log.error(error);
            }
            return data;
        });
    }
    monitor(change) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const stream = yield this.docker.getEvents();
                stream.on('data', (chunk) => {
                    const event = JSON.parse(chunk.toString());
                    log_1.Log.info(event);
                    if (event.Type === 'container' && (event.Action === 'start' || event.Action === 'stop'
                        || event.Action === 'create' || event.Action === 'destroy' || event.Action === 'rename'
                        || event.Action === 'die')) {
                        change();
                    }
                });
            }
            catch (error) {
                log_1.Log.error(error);
            }
        });
    }
}
exports.Dockerode = Dockerode;
